#include "AudioPlayerComponent.h"
#include "../core/utils/Logging.h"

AudioPlayerComponent::AudioPlayerComponent(Object* owner)
    : Component(owner) {}

AudioPlayerComponent::~AudioPlayerComponent()
{
    if (m_OwnsSound && m_Sound)
    {
        ma_sound_uninit(m_Sound);
        delete m_Sound;
    }
}

void AudioPlayerComponent::SetAudio(uint64_t uaid)
{
    Stop();
    m_UAID = uaid;

    const auto* asset = dynamic_cast<const AudioAssetInfo*>(AssetManager::GetAssetByID(uaid));
    if (!asset || !asset->loaded)
    {
        Logger::LogWarning("AudioPlayerComponent: Asset not found or not loaded (UAID: %llu)", uaid);
        m_Sound = nullptr;
        return;
    }

    m_Sound = asset->sound;
    m_OwnsSound = false;

    ma_sound_set_volume(m_Sound, volume);
    ma_sound_set_looping(m_Sound, loop ? MA_TRUE : MA_FALSE);
}

void AudioPlayerComponent::Play()
{
    if (m_Sound) ma_sound_start(m_Sound);
}

void AudioPlayerComponent::Pause()
{
    if (m_Sound) ma_sound_stop(m_Sound);
}

void AudioPlayerComponent::Stop()
{
    if (m_Sound)
    {
        ma_sound_stop(m_Sound);
        ma_sound_seek_to_pcm_frame(m_Sound, 0);
    }
}

void AudioPlayerComponent::SetLooping(bool value)
{
    loop = value;
    if (m_Sound) ma_sound_set_looping(m_Sound, value ? MA_TRUE : MA_FALSE);
}

void AudioPlayerComponent::SetVolume(float vol)
{
    volume = vol;
    if (m_Sound) ma_sound_set_volume(m_Sound, vol);
}

bool AudioPlayerComponent::IsPlaying() const
{
    return m_Sound && ma_sound_is_playing(m_Sound);
}

std::string AudioPlayerComponent::GetName() const
{
    return "AudioPlayerComponent";
}

void AudioPlayerComponent::Save(YAML::Emitter& out) const
{
    
    out << YAML::BeginMap;
    out << YAML::Key << "type" << YAML::Value << GetName();
    out << YAML::Key << "uaid" << YAML::Value << m_UAID;
    out << YAML::Key << "volume" << YAML::Value << volume;
    out << YAML::Key << "loop" << YAML::Value << loop;
    out << YAML::EndMap;

}

void AudioPlayerComponent::Load(const YAML::Node& node)
{
    if (node["uaid"])
    {
        m_UAID = node["uaid"].as<uint64_t>();
        SetAudio(m_UAID);
    }
    if (node["volume"])
        SetVolume(node["volume"].as<float>());
    if (node["loop"])
        SetLooping(node["loop"].as<bool>());
}
