#define MA_ENABLE_MP3
#define MA_ENABLE_WAV
#define MA_ENABLE_FLAC
#define MA_ENABLE_VORBIS
#define MA_IMPLEMENTATION
#include "AudioEngine.h"
#include "../utils/AssetManager.h"
#include "../utils/Logging.h"

ma_engine AudioEngine::s_Engine{};
std::unordered_map<uint64_t, PlayingSound> AudioEngine::s_PlayingSounds;

bool AudioEngine::Init() {
    if (ma_engine_init(nullptr, &s_Engine) != MA_SUCCESS) {
        Logger::LogError("Failed to start AudioCore");
        return false;
    }
    Logger::LogVerbose("AudioCore initialized successfully.");
    return true;
}

void AudioEngine::Shutdown() {
    for (auto& [_, s] : s_PlayingSounds)
        s.Stop();
    s_PlayingSounds.clear();
    ma_engine_uninit(&s_Engine);
}

void AudioEngine::Play(uint64_t uaid, bool loop) {
    if (s_PlayingSounds.find(uaid) == s_PlayingSounds.end()) {
        const auto* asset = AssetManager::GetAssetByID(uaid);
        if (!asset || !asset->loaded || asset->type != AssetType::Audio) {
            Logger::LogError("[AudioEngine] Invalid or unloaded audio asset: %llu", uaid);
            return;
        }

        ma_sound sound;
        if (ma_sound_init_from_file(&s_Engine, asset->path.c_str(), 0, nullptr, nullptr, &sound) != MA_SUCCESS) {
            Logger::LogError("[AudioEngine] Failed to load sound: %s", asset->path.c_str());
            return;
        }

        ma_sound_set_looping(&sound, loop);

        ma_uint64 totalFrames = 0;
        ma_sound_get_length_in_pcm_frames(&sound, &totalFrames);
        ma_uint32 sampleRate = ma_engine_get_sample_rate(&s_Engine);
        double totalSeconds = sampleRate > 0 ? double(totalFrames) / sampleRate : 0.0;

        PlayingSound ps{ uaid, asset->filename, sound, 0.0, totalSeconds };
        s_PlayingSounds[uaid] = ps;

        ma_sound_start(&s_PlayingSounds[uaid].sound);
    } else {
        auto& s = s_PlayingSounds[uaid];
        ma_sound_set_looping(&s.sound, loop);
        ma_sound_start(&s.sound);
    }
}

void AudioEngine::Stop(uint64_t uaid) {
    auto it = s_PlayingSounds.find(uaid);
    if (it != s_PlayingSounds.end()) {
        it->second.Stop();
        s_PlayingSounds.erase(it);
    }
}

void AudioEngine::StopAll() {
    for (auto& [_, s] : s_PlayingSounds)
        s.Stop();
    s_PlayingSounds.clear();
}

void AudioEngine::Unload(uint64_t uaid) {
    Stop(uaid);
}

void AudioEngine::Update() {
    for (auto it = s_PlayingSounds.begin(); it != s_PlayingSounds.end();) {
        if (!ma_sound_is_playing(&it->second.sound)) {
            it->second.Stop();
            it = s_PlayingSounds.erase(it);
        } else {
            ma_uint64 cursorFrames = 0;
            ma_sound_get_cursor_in_pcm_frames(&it->second.sound, &cursorFrames);
            ma_uint32 sampleRate = ma_engine_get_sample_rate(&s_Engine);
            it->second.currentTime = sampleRate > 0 ? static_cast<double>(cursorFrames) / sampleRate : 0.0;
            ++it;
        }
    }
}

const std::unordered_map<uint64_t, PlayingSound>& AudioEngine::GetPlayingSounds() {
    return s_PlayingSounds;
}
